==> local-booter-9.7.8-sources.jar: com/xebialabs/deployit/booter/local/LocalCreatorDescriptor.java <==
package com.xebialabs.deployit.booter.local;

import com.xebialabs.deployit.booter.local.utils.ReflectionUtils;
import com.xebialabs.deployit.plugin.api.creator.CreatorContext;
import com.xebialabs.deployit.plugin.api.reflect.CreatorDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.Descriptor;
import com.xebialabs.deployit.plugin.api.udm.Delegate;
import org.w3c.dom.Attr;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import static com.xebialabs.deployit.plugin.api.udm.ControlTask.DEFAULT_DELEGATE;
import static com.xebialabs.deployit.booter.local.utils.XmlUtils.getRequiredStringAttribute;
import static com.xebialabs.overthere.util.OverthereUtils.checkNotNull;

class LocalCreatorDescriptor implements CreatorDescriptor {

    private String name;
    private String delegate;
    private Map<String, String> attributes = new HashMap<>();
    private Descriptor descriptor;

    private LocalCreatorDescriptor(Descriptor descriptor, String name) {
        this.name = name;
        this.descriptor = descriptor;
    }

    private LocalCreatorDescriptor(Descriptor descriptor, Method method) {
        this.name = method.getName();
        this.delegate = DEFAULT_DELEGATE;
        this.descriptor = descriptor;
    }

    LocalCreatorDescriptor(LocalCreatorDescriptor copyOf, Descriptor newOwner) {
        this.attributes = copyOf.attributes;
        this.delegate = copyOf.delegate;
        this.descriptor = newOwner;
    }

    static CreatorDescriptor from(Descriptor descriptor, Method method) {
        LocalCreatorDescriptor creatorDescriptor = new LocalCreatorDescriptor(descriptor, method);
        initVerifications(creatorDescriptor);
        return creatorDescriptor;
    }

    static CreatorDescriptor from(Descriptor descriptor, Element element) {
        String delegate = getRequiredStringAttribute(element, "delegate");
        LocalCreatorDescriptor creatorDescriptor = new LocalCreatorDescriptor(descriptor, delegate);
        creatorDescriptor.delegate = delegate;
        creatorDescriptor.attributes = readAttributes(element);
        initVerifications(creatorDescriptor);
        return creatorDescriptor;
    }

    private static void initVerifications(final LocalCreatorDescriptor creatorDescriptor) {
        checkNotNull(DelegateRegistry.getDelegate(creatorDescriptor.delegate, Delegate.Use.CREATOR), creatorDescriptor.delegate + " is referenced, but not registered.");
    }

    private static Map<String, String> readAttributes(final Element element) {
        Map<String, String> map = new HashMap<>();
        NamedNodeMap xmlAttrs = element.getAttributes();
        for (int i = 0; i < xmlAttrs.getLength(); i++) {
            Attr item = (Attr) xmlAttrs.item(i);
            map.put(item.getName(), item.getValue());
        }
        return map;
    }

    void verify(final Verifications verifications) {
        verifications.verify(descriptor.getType(), DelegateRegistry.exists(delegate, Delegate.Use.CREATOR), "No delegate called [%s] available for creator [%s]", delegate, getFqn());
    }

    Descriptor getDescriptor() {
        return descriptor;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public Map<String, String> getAttributes() {
        return attributes;
    }

    @Override
    public void invoke(CreatorContext context) {
        this.invokeDelegate(context);
    }

    @SuppressWarnings("unchecked")
    private void invokeDelegate(CreatorContext context) {
        Method method = DelegateRegistry.getDelegate(delegate, Delegate.Use.CREATOR);
        try {
            if (method.getParameterCount() == 2) {
                method.invoke(null, context, attributes);
            } else {
                method.invoke(null, context);
            }
        } catch (IllegalAccessException e) {
            throw new RuntimeException("Could not invoke " + delegate + " on " + context.getThisCI());
        } catch (InvocationTargetException e) {
            throw ReflectionUtils.handleInvocationTargetException(e, "Could not invoke " + delegate + " on " + context.getThisCI());
        }
    }

    @Override
    public String getFqn() {
        return String.format("%s.%s", descriptor.getType(), name);
    }

}

==> local-booter-22.3.10-sources.jar: com/xebialabs/deployit/booter/local/LocalCreatorDescriptor.java <==
package com.xebialabs.deployit.booter.local;

import com.xebialabs.deployit.booter.local.utils.ReflectionUtils;
import com.xebialabs.deployit.plugin.api.creator.CreatorContext;
import com.xebialabs.deployit.plugin.api.reflect.CreatorDescriptor;
import com.xebialabs.deployit.plugin.api.reflect.Descriptor;
import com.xebialabs.deployit.plugin.api.udm.Delegate;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Map;

import static com.xebialabs.deployit.plugin.api.udm.ControlTask.DEFAULT_DELEGATE;
import static com.xebialabs.overthere.util.OverthereUtils.checkNotNull;

public class LocalCreatorDescriptor implements CreatorDescriptor {

    private String name;
    private String delegate;
    private Map<String, String> attributes = new HashMap<>();
    private Descriptor descriptor;

    private LocalCreatorDescriptor(Descriptor descriptor, String name) {
        this.name = name;
        this.descriptor = descriptor;
    }

    private LocalCreatorDescriptor(Descriptor descriptor, Method method) {
        this.name = method.getName();
        this.delegate = DEFAULT_DELEGATE;
        this.descriptor = descriptor;
    }

    LocalCreatorDescriptor(LocalCreatorDescriptor copyOf, Descriptor newOwner) {
        this.attributes = copyOf.attributes;
        this.delegate = copyOf.delegate;
        this.descriptor = newOwner;
    }

    static CreatorDescriptor from(Descriptor descriptor, Method method) {
        LocalCreatorDescriptor creatorDescriptor = new LocalCreatorDescriptor(descriptor, method);
        initVerifications(creatorDescriptor);
        return creatorDescriptor;
    }

    public static CreatorDescriptor from(LocalDescriptor descriptor, String delegate, Map<String, String> attributes) {
        LocalCreatorDescriptor creatorDescriptor = new LocalCreatorDescriptor(descriptor, delegate);
        creatorDescriptor.delegate = delegate;
        creatorDescriptor.attributes = attributes;
        initVerifications(creatorDescriptor);
        return creatorDescriptor;
    }

    private static void initVerifications(final LocalCreatorDescriptor creatorDescriptor) {
        checkNotNull(DelegateRegistry.getDelegate(creatorDescriptor.delegate, Delegate.Use.CREATOR), creatorDescriptor.delegate + " is referenced, but not registered.");
    }

    void verify(final Verifications verifications) {
        verifications.verify(descriptor.getType(), DelegateRegistry.exists(delegate, Delegate.Use.CREATOR), "No delegate called [%s] available for creator [%s]", delegate, getFqn());
    }

    Descriptor getDescriptor() {
        return descriptor;
    }

    @Override
    public String getName() {
        return name;
    }

    @Override
    public Map<String, String> getAttributes() {
        return attributes;
    }

    @Override
    public void invoke(CreatorContext context) {
        this.invokeDelegate(context);
    }

    @SuppressWarnings("unchecked")
    private void invokeDelegate(CreatorContext context) {
        Method method = DelegateRegistry.getDelegate(delegate, Delegate.Use.CREATOR);
        try {
            if (method.getParameterCount() == 2) {
                method.invoke(null, context, attributes);
            } else {
                method.invoke(null, context);
            }
        } catch (IllegalAccessException e) {
            throw new RuntimeException("Could not invoke " + delegate + " on " + context.getThisCI());
        } catch (InvocationTargetException e) {
            throw ReflectionUtils.handleInvocationTargetException(e, "Could not invoke " + delegate + " on " + context.getThisCI());
        }
    }

    @Override
    public String getFqn() {
        return String.format("%s.%s", descriptor.getType(), name);
    }

}
